@@ -6,8 +6,8 @@
 }
 
 .identical_h5loc <- function(x, y) {
-  identical(x = x$get_filename(), y = y$get_filename()) &&
-    identical(x = x$get_obj_name(), y = y$get_obj_name())
+  (x$get_filename() == y$get_filename()) & 
+    (x$get_obj_name() == y$get_obj_name())
 }
 
 ## Exclude hdf5 link names =====================================================
@@ -134,7 +134,7 @@
     )
   }
   is.identical <- .identical_h5loc(x = from.h5obj, y = to.h5obj) &&
-    identical(x = from.which, y = to.which)
+    (from.which == to.which)
   if (is.identical) {
     warning(
       "Source attribute is identical to the destination, skip copying: ",
@@ -206,14 +206,14 @@
     verbose = TRUE,
     ...
 ) {
-  if (identical(x = from.name, y = to.name)) {
+  if (from.name == to.name) {
     warning(
       "The source object is identical to the destination, skip copying.",
       immediate. = TRUE
     )
     return(invisible(x = NULL))
   }
-  if (identical(x = to.name, y = "/")) {
+  if (to.name == "/") {
     stop("\nCannot copy object to '/' within an H5 file.")
   }
   h5fh <- h5TryOpen(filename = h5.file, mode = "r+")
@@ -260,7 +260,7 @@
     verbose = TRUE,
     ...
 ) {
-  if (!identical(x = to.name, y = "/")) {
+  if (to.name != "/") {
     to.h5fh <- h5TryOpen(filename = to.file, mode = "a")
     if (h5Exists(x = to.h5fh, name = to.name)) {
       if (!overwrite) {
@@ -283,7 +283,7 @@
       )
       return(invisible(x = NULL))
     }
-    if (identical(x = from.name, y = "/")) {
+    if (from.name == "/") {
       verboseMsg("Copy the source file directly.")
       file.copy(from = from.file, to = to.file, overwrite = TRUE)
       return(invisible(x = NULL))
@@ -528,13 +528,7 @@
   ))
 }
 
-.h5group_read <- function(
-    h5group, 
-    name = NULL, 
-    transpose = FALSE, 
-    toS4.func = NULL, 
-    ...
-) {
+.h5group_read <- function(h5group, name = NULL, transpose = FALSE, ...) {
   if (!is.null(x = name)) {
     h5obj <- h5Open(x = h5group, name = name)
     if (!identical(x = h5obj, y = h5group)) {
@@ -543,16 +537,11 @@
     if (inherits(x = h5obj, what = "H5D")) {
       return(h5ReadDataset(x = h5obj, transpose = transpose))
     }
-    return(h5Read(
-      x = h5obj, 
-      transpose = transpose, 
-      toS4.func = toS4.func, 
-      ...
-    ))
+    return(h5Read(x = h5obj, transpose = transpose, ...))
   }
   encoding_type <- h5Attr(x = h5group, which = "encoding-type")
   encoding_type <- encoding_type %||% ""
-  r_obj <- switch(
+  robj <- switch(
     EXPR = encoding_type,
     "categorical" = .h5read_factor(h5obj = h5group),
     "dataframe" = .h5read_dataframe(h5obj = h5group),
@@ -562,17 +551,9 @@
     "nullable-integer" = .h5read_nullable(h5obj = h5group),
     .h5read_list(h5obj = h5group, transpose = transpose, ...)
   )
-  if (is.null(x = toS4.func)) {
-    return(r_obj)
-  }
-  if (!is.function(x = toS4.func)) {
-    stop("\n  'toS4.func' must be NULL or a function.")
-  }
-  S4Class <- h5Attr(x = h5group, which = "S4Class")
-  return(toS4.func(r_obj, S4Class))
+  return(robj)
 }
 
-
 ## Assertive helpers ===========================================================
 #' @importFrom hdf5r H5D
 .assert_h5d <- function(h5obj) {
@@ -625,7 +606,7 @@
   n.h5dims <- length(x = maxdims)
   name <- h5obj$get_obj_name()
   if (is.null(x = idx_list)) {
-    if (!identical(x = n.dims, y = n.h5dims)) {
+    if (n.dims != n.h5dims) {
       stop(
         "\nDimension number doesn't match: ",
         "\n  Input R object: ", n.dims,
@@ -648,7 +629,7 @@
   if (!is.list(x = idx_list)) {
     stop("\n  'idx_list' must be either NULL or a list")
   }
-  if (!identical(x = length(x = idx_list), y = n.h5dims)) {
+  if (length(x = idx_list) != n.h5dims) {
     stop(
       "\nElement number of 'idx_list' doesn't match ",
       "the dimension number of dataset:",
@@ -663,7 +644,7 @@
     FUN.VALUE = integer(length = 1)
   )
   len.idx <- lengths(x = idx_list)
-  if (!identical(x = len.idx, y = dims)) {
+  if (any(dims != len.idx)) {
     stop(
       "\n'idx_list' doesn't match the dimensions of 'robj':",
       "\n  idx_list: ", paste(len.idx, collapse = ", "),
@@ -691,14 +672,25 @@
   return(invisible(x = NULL))
 }
 
+.h5write_add_extra <- function(extra, h5group, name, ...) {
+  h5obj <- h5Open(x = h5group, name = name)
+  if (!inherits(x = h5obj, what = "H5File")) {
+    on.exit(expr = h5obj$close())
+  }
+  for (i in names(x = extra)) {
+    .h5attr_write(h5obj = h5obj, which = i, robj = extra[[i]], ...)
+  }
+  return(invisible(x = NULL))
+}
+
 .h5d_set_encode <- function(robj, h5d, ...) {
   type.x <- typeof(x = robj)
-  .h5attr_write(h5obj = h5d, which = "encoding-version", robj = "0.2.0")
   encoding_type <- "array"
   if (is.character(x = robj)) {
     encoding_type <- "string-array"
   }
   .h5attr_write(h5obj = h5d, which = "encoding-type", robj = encoding_type)
+  .h5attr_write(h5obj = h5d, which = "encoding-version", robj = "0.2.0")
   return(invisible(x = NULL))
 }
 
@@ -746,6 +738,34 @@
   return(invisible(x = NULL))
 }
 
+.h5group_write_vector <- function(
+    robj, 
+    h5group, 
+    name, 
+    stype = c('utf8', 'ascii7'), 
+    ...
+) {
+  if (length(x = robj) == 0) {
+    return(invisible(x = NULL))
+  }
+  stype <- match.arg(arg = stype)
+  dims <- .get_valid_dims(x = robj, transpose = FALSE)
+  h5CreateDataset(
+    x = h5group,
+    name = name,
+    dtype = NULL,
+    storage.mode = robj[1],
+    stype = stype,
+    dims = dims,
+    ...
+  )
+  h5d <- h5Open(x = h5group, name = name)
+  on.exit(expr = h5d$close(), add = TRUE)
+  h5d$write(args = NULL, value = as.array(x = robj))
+  .h5d_set_encode(robj = robj, h5d = h5d)
+  return(invisible(x = NULL))
+}
+
 .h5write_vector <- function(
     x, 
     file, 
@@ -761,11 +781,18 @@
   file <- h5Overwrite(file = file, name = name, overwrite = overwrite)
   h5fh <- h5TryOpen(filename = file, mode = "r+")
   on.exit(expr = h5fh$close())
-  .h5write_array(
-    robj = x, 
-    h5group = h5fh, 
-    name = name, 
-    transpose = FALSE, 
+  # .h5write_array(
+  #   robj = x, 
+  #   h5group = h5fh, 
+  #   name = name, 
+  #   transpose = FALSE, 
+  #   gzip_level = gzip_level,
+  #   ...
+  # )
+  .h5group_write_vector(
+    robj = x,
+    h5group = h5fh,
+    name = name,
     gzip_level = gzip_level,
     ...
   )
@@ -786,28 +813,20 @@
     categories = levels(x = robj)
   )
   for (i in names(x = r_obj)) {
-    .h5write_array(
+    .h5group_write_vector(
       robj = r_obj[[i]],
       h5group = h5group,
       name = file.path(name, i),
-      transpose = FALSE,
       gzip_level = gzip_level,
       ...
     )
   }
-  h5WriteAttr(x = h5group, name = name, which = "ordered", robj = ordered)
-  h5WriteAttr(
-    x = h5group, 
-    name = name, 
-    which = "encoding-type", 
-    robj = "categorical"
-  )
-  h5WriteAttr(
-    x = h5group, 
-    name = name,
-    which = "encoding-version",
-    robj = "0.2.0"
+  extra <- list(
+    "encoding-type" = "categorical", 
+    "encoding-version" = "0.2.0",
+    "ordered" = ordered
   )
+  .h5write_add_extra(extra = extra, h5group = h5group, name = name)
   return(invisible(x = NULL))
 }
 
@@ -819,33 +838,19 @@
     gzip_level = 6, 
     ...
 ) {
-  .h5write_array(
+  .h5group_write_vector(
     robj = rownames(x = robj), 
     h5group = h5group, 
     name = file.path(name, "_index"), 
-    transpose = FALSE, 
-    maxdims = nrow(x = robj),
     gzip_level = gzip_level,
     ...
   )
-  h5WriteAttr(
-    x = h5group, 
-    name = name, 
-    which = "encoding-type",
-    robj = "dataframe"
-  )
-  h5WriteAttr(
-    x = h5group, 
-    name = name, 
-    which = "encoding-version",
-    robj = "0.2.0"
-  )
-  h5WriteAttr(
-    x = h5group, 
-    name = name, 
-    which = "_index",
-    robj = "_index"
+  extra <- list(
+    "encoding-type" = "dataframe", 
+    "encoding-version" = "0.2.0",
+    "_index" = "_index"
   )
+  .h5write_add_extra(extra = extra, h5group = h5group, name = name)
   if (ncol(x = robj) == 0) {
     ## Must add an empty 'column-order' to match anndata's IOSpec
     h5obj <- h5group$open(name = name)
@@ -896,7 +901,7 @@
     h5group, 
     name, 
     add.shape = FALSE, 
-    dimnames = list(),
+    add.dimnames = TRUE,
     gzip_level = gzip_level,
     ...
 ) {
@@ -905,11 +910,10 @@
     h5d_sparse_names <- c(x = "data", j = "indices", p = "indptr")
   }
   for (i in names(x = h5d_sparse_names)) {
-    .h5write_array(
+    .h5group_write_vector(
       robj = slot(object = robj, name = i),
       h5group = h5group,
       name = file.path(name, h5d_sparse_names[i]),
-      transpose = FALSE,
       maxdims = Inf,
       gzip_level = gzip_level,
       ...
@@ -920,7 +924,7 @@
     h5group = h5group, 
     name = name, 
     add.shape = add.shape, 
-    dimnames = dimnames
+    add.dimnames = add.dimnames
   )
   return(invisible(x = NULL))
 }
@@ -930,47 +934,34 @@
     h5group, 
     name,
     add.shape = FALSE, 
-    dimnames = list()
+    add.dimnames = TRUE
 ) {
-  h5WriteAttr(
-    x = h5group,
-    name = name, 
-    which = "shape",
-    robj = rev(x = dim(x = robj))
-  )
   encode_type <- if (is(object = robj, class2 = "dgCMatrix")) {
     "csr_matrix"
   } else if (is(object = robj, class2 = "dgRMatrix")) {
     "csc_matrix"
   }
-  h5WriteAttr(
-    x = h5group,
-    name = name, 
-    which = "encoding-type",
-    robj = encode_type
-  )
-  h5WriteAttr(
-    x = h5group,
-    name = name, 
-    which = "encoding-version",
-    robj = "0.1.0"
+  extra <- list(
+    "encoding-type" = encode_type,
+    "encoding-version" = "0.1.0",
+    "shape" = rev(x = dim(x = robj))
   )
+  .h5write_add_extra(extra = extra, h5group = h5group, name = name)
   if (add.shape) {
-    .h5write_array(
+    .h5group_write_vector(
       robj = rev(x = dim(x = robj)), 
       h5group = h5group, 
-      name = file.path(name, "shape"), 
-      transpose = FALSE
+      name = file.path(name, "shape")
     )
   }
-  if (identical(x = lengths(x = dimnames), y = dim(x = robj))) {
+  if (add.dimnames) {
+    dimnames <- dimnames(x = robj)
     names(x = dimnames) <- c("row_names", "col_names")
     for (i in names(x = dimnames)) {
-      .h5write_array(
+      .h5group_write_vector(
         robj = dimnames[[i]], 
         h5group = h5group, 
-        name = file.path(name, i),
-        transpose = FALSE
+        name = file.path(name, i)
       )
     }
   }
@@ -988,7 +979,7 @@
     stop("\n  'idx_list' must be either NULL or a list")
   }
   dims <- h5obj$dims
-  if (!identical(x = length(x = idx_list), y = length(x = dims))) {
+  if (length(x = idx_list) != length(x = dims)) {
     stop(
       "\nElement number of 'idx_list' doesn't match ",
       "the dimension number of dataset:",
@@ -1054,7 +1045,7 @@
 
 .h5read_dataframe <- function(h5obj) {
   col_orders <- .h5get_column_order(h5obj = h5obj)
-  index <- h5Attr(x = h5obj, which = "_index")
+  index <- .h5attr(h5obj = h5obj, which = "_index")
   r_list <- .h5read_list(h5obj = h5obj)
   rownames <- NULL
   if (length(x = index) > 0) {
@@ -1087,7 +1078,7 @@
 #' @importFrom hdf5r h5garbage_collect
 #' @importMethodsFrom Matrix t
 .h5read_sparse <- function(h5obj, transpose = FALSE) {
-  dims <- h5Attr(x = h5obj, which = "shape")
+  dims <- .h5attr(h5obj = h5obj, which = "shape")
   h5data <- h5obj$open(name = "data")
   on.exit(expr = h5data$close(), add = TRUE)
   h5indices <- h5obj$open(name = "indices")
